@@ -34,21 +34,22 @@
 
 #define REMAP_PAGE_RANGE_FN remap_pfn_range
 #define REMAP_PAGE_RANGE_STR "remap_pfn_range"
-#define REMAP_PAGE_RANGE_OFF(vma) ((vma)->vm_pgoff)
+#define REMAP_PAGE_RANGE_OFF(offset) ((offset) >> PAGE_SHIFT)
 
 #else /* LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,9) */
 
 #define REMAP_PAGE_RANGE_FN remap_page_range
 #define REMAP_PAGE_RANGE_STR "remap_page_range"
-#define REMAP_PAGE_RANGE_OFF(vma) __ke_vm_offset(vma)
+#define REMAP_PAGE_RANGE_OFF(offset) (offset)
 
 #endif /* LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9) */
 
-#define REMAP_PAGE_RANGE(vma) REMAP_PAGE_RANGE_FN(FGL_VMA_API_PASS \
-                                                  (vma)->vm_start,	\
-                                                  REMAP_PAGE_RANGE_OFF(vma), \
-                                                  (vma)->vm_end - (vma)->vm_start, \
-                                                  (vma)->vm_page_prot)
+#define REMAP_PAGE_RANGE(vma,offset) \
+    REMAP_PAGE_RANGE_FN(FGL_VMA_API_PASS \
+                        (vma)->vm_start,	\
+                        REMAP_PAGE_RANGE_OFF(offset), \
+                        (vma)->vm_end - (vma)->vm_start, \
+                        (vma)->vm_page_prot)
 
 
 /* Page table macros */
@@ -152,10 +153,38 @@ do { \
     } \
 } while(0)
 
-#ifdef __x86_64__
-#define HAS_PTE_HIGH_ADDR(pte) 0
+#ifdef pfn_to_page
+#define PMD_PAGE(pmd) pmd_page(pmd)
+#else /* for old 2.4 kernels */
+/* on 2.6 x86_64 kernel, pfn_to_page may not be defined in some cases (like with NUMA option on). 
+   This is intended, we should not redefine it to break the build.
+*/
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
+#define pfn_to_page(pfn) (mem_map + (pfn))
+#endif
+#define PMD_PAGE(pmd) (pfn_to_page(pmd_val(pmd) >> PAGE_SHIFT))
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
+#if !defined(CONFIG_SMP) || defined(CONFIG_SUSPEND_SMP) // ACPI not working on older SMP kernel (prior to 2.6.13) 
+#define FIREGL_POWER_MANAGEMENT
+#endif
+#endif
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,14)
+#define PMSG_EVENT(pmsg_state) (pmsg_state).event
 #else
-#define HAS_PTE_HIGH_ADDR(pte) ((((u64)pte_val(pte) >> 32) & 0xf) ? 1:0)
+#define PMSG_EVENT(pmsg_state) (pmsg_state)
+/* For old kernels without PM_EVENT_xxx defines, define them 
+   in consistent with the power state used in these kernels.
+ */
+#define PM_EVENT_ON 0
+#define PM_EVENT_SUSPEND 3
+#define PM_EVENT_FREEZE 3
+#endif
+
+#if defined(CONFIG_COMPAT) && LINUX_VERSION_CODE > KERNEL_VERSION(2,6,9)
+#define FIREGL_IOCTL_COMPAT
 #endif
 
 /*****************************************************************************/
@@ -199,8 +228,9 @@ typedef struct { int uniqe2; } __ke_wait_queue_t;
 typedef struct { int uniqe3; } __ke_sigset_t;
 typedef struct { int uniqe4; } __ke_pci_dev_t;
 typedef struct { int uniqe5; } __ke_priv_device_t;
+typedef struct { int uniqe6; } __ke_pci_bus_t;
 
-typedef	int (*ATI_API_CALL __ke_read_proc_t)(
+typedef	int (*__ke_read_proc_t)(
     char* page, char** start, __ke_off_t off, int count, int* eof, void* data);
 
 typedef struct {
@@ -208,6 +238,8 @@ typedef struct {
     __ke_read_proc_t    f;
 } __ke_proc_list_t;
 
+extern __ke_proc_list_t firegl_proc_list[];
+
 typedef struct {
     unsigned long           signature;
     __ke_priv_device_t *    privdev;
@@ -230,6 +262,22 @@ typedef struct __ke_pci_device_info
 
 /*****************************************************************************/
 
+extern int ATI_API_CALL drm_name_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_bios_version(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_interrupt_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL drm_mem_info(char* buf, char** start, __ke_off_t offset, int len, int *eof, void *data);
+extern int ATI_API_CALL drm_mem_info1(char* buf, char** start, __ke_off_t offset, int len, int *eof, void *data);
+extern int ATI_API_CALL drm_vm_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL drm_clients_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_lock_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_umm_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+#ifdef DEBUG
+extern int ATI_API_CALL drm_bq_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+extern int ATI_API_CALL firegl_debug_info(char* buf, char** start, __ke_off_t offset, int len, int* eof, void* data);
+#endif
+
+/*****************************************************************************/
+
 extern int ATI_API_CALL firegl_init(__ke_device_t*);
 extern void ATI_API_CALL firegl_cleanup(__ke_device_t*);
 extern int ATI_API_CALL firegl_open(struct inode* inode, struct file* filp);
@@ -239,6 +287,13 @@ extern int ATI_API_CALL firegl_ioctl(struct inode* inode,
                         unsigned int cmd,
                         unsigned long arg);
 
+#ifdef __x86_64__
+extern long ATI_API_CALL firegl_compat_ioctl(
+                        struct file* filp,
+                        unsigned int cmd,
+                        unsigned long arg);
+#endif
+
 /*****************************************************************************/
 
 extern int ATI_API_CALL firegl_mmap(struct file* filp, struct vm_area_struct* vma);
@@ -258,6 +313,8 @@ extern void* ATI_API_CALL firegl_get_pciemem_from_addr( struct vm_area_struct* v
 extern unsigned long ATI_API_CALL firegl_get_pcie_pageaddr_from_vm(  struct vm_area_struct* vma, struct firegl_pcie_mem* pciemem, unsigned long offset);
 extern void* ATI_API_CALL firegl_get_pagelist_from_vm(  struct vm_area_struct* vma );
 extern unsigned long ATI_API_CALL firegl_get_addr_from_vm(  struct vm_area_struct* vma);
+extern unsigned long ATI_API_CALL firegl_get_pagetable_page_from_vm(struct vm_area_struct* vma);
+
 
 /*****************************************************************************/
 
@@ -269,6 +326,8 @@ extern void ATI_API_CALL __ke_free_wait_queue_struct(__ke_wait_queue_t* queue);
 extern void ATI_API_CALL __ke_wake_up_interruptible(__ke_wait_queue_head_t* queue_head);
 extern void ATI_API_CALL __ke_add_wait_queue(__ke_wait_queue_head_t* queue_head, __ke_wait_queue_t* entry);
 extern void ATI_API_CALL __ke_remove_wait_queue(__ke_wait_queue_head_t* queue_head, __ke_wait_queue_t* entry);
+extern void ATI_API_CALL __ke_init_waitqueue_head(__ke_wait_queue_head_t* queue_head);
+extern void ATI_API_CALL __ke_wait_event_interruptible(__ke_wait_queue_head_t* queue_head, int condition);
 
 extern void ATI_API_CALL __ke_schedule(void);
 extern int ATI_API_CALL __ke_signal_pending(void);
@@ -277,7 +336,9 @@ extern void ATI_API_CALL __ke_yield(void);
 extern void ATI_API_CALL __ke_set_current_state_task_interruptible(void);
 extern void ATI_API_CALL __ke_set_current_state_task_running(void);
 extern void ATI_API_CALL __ke_configure_sigmask(__ke_sigset_t *pSigMask);
-extern void ATI_API_CALL __ke_block_all_signals(int (*ATI_API_CALL notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask);
+extern int ATI_API_CALL firegl_sig_notifier(void *priv);
+extern int              firegl_sig_notifier_wrap(void *priv);
+extern void ATI_API_CALL __ke_block_all_signals(int (*notifier)(void *priv), void *pPriv, __ke_sigset_t *pSigMask);
 extern void ATI_API_CALL __ke_unblock_all_signals(void);
 
 extern unsigned long ATI_API_CALL __ke__cmpxchg(volatile void *ptr, unsigned long old,                      
@@ -298,6 +359,7 @@ extern void ATI_API_CALL __ke_set_file_priv(struct file* filp, void* private_dat
 extern int ATI_API_CALL __ke_file_excl_open(struct file* filp);
 extern int ATI_API_CALL __ke_file_rw_open(struct file* filp);
 extern unsigned int ATI_API_CALL __ke_file_counter(struct file* filp);
+extern struct inode* ATI_API_CALL __ke_get_file_inode(struct file* filp);
 extern int ATI_API_CALL __ke_getpid(void);
 extern int ATI_API_CALL __ke_geteuid(void);
 extern unsigned long ATI_API_CALL __ke_jiffies(void);
@@ -355,6 +417,8 @@ extern int ATI_API_CALL __ke_sys_mlock(unsigned long start, __ke_size_t len);
 extern int ATI_API_CALL __ke_sys_mlock(unsigned long start, __ke_size_t len);
 extern int ATI_API_CALL __ke_sys_munlock(unsigned long start, __ke_size_t len);
 extern int ATI_API_CALL __ke_sys_modify_ldt(int func, void *ptr, unsigned long bytecount);
+int ATI_API_CALL __ke_vsprintf(char *buf, const char *fmt, va_list ap);
+int ATI_API_CALL __ke_vsnprintf(char *buf, size_t size, const char *fmt, va_list ap);
 #ifdef __KE_NO_VSPRINTF
 extern void ATI_API_CALL __ke_printk(const char* fmt, ...);
 #else // !__KE_NO_VSPRINTF
@@ -374,6 +438,9 @@ extern __ke_u32 ATI_API_CALL __ke_get_cap_effective(void);
 extern void ATI_API_CALL __ke_set_cap_effective(__ke_u32 cap);
 extern unsigned long ATI_API_CALL __ke_ram_available(void);
 
+#ifdef __x86_64__
+extern void* ATI_API_CALL __ke_compat_alloc_user_space(long size);
+#endif
 extern int ATI_API_CALL __ke_copy_from_user(void* to, const void* from, __ke_size_t size);
 extern int ATI_API_CALL __ke_copy_to_user(void* to, const void* from, __ke_size_t size);
 extern int ATI_API_CALL __ke_verify_area(int type, const void * addr, unsigned long size);
@@ -405,7 +472,8 @@ extern int ATI_API_CALL __ke_verify_write_access(void* addr, __ke_size_t size);
 extern struct ATI_API_CALL mm_struct* ATI_API_CALL __ke_init_mm(void);
 extern struct ATI_API_CALL mm_struct* ATI_API_CALL __ke_current_mm(void);
 extern __ke_dma_addr_t ATI_API_CALL __ke_get_vm_phys_addr(struct mm_struct* mm, unsigned long virtual_addr);
-extern unsigned long ATI_API_CALL __ke_get_vm_page_table(struct mm_struct* mm, unsigned long virtual_addr);
+extern unsigned long ATI_API_CALL __ke_get_vm_page_table(struct mm_struct* mm, unsigned long virtual_addr, unsigned long *page_addr, int* is_highpte);
+extern void ATI_API_CALL __ke_put_vm_page_table(unsigned long page_addr);
 extern __ke_dma_addr_t* ATI_API_CALL __ke_get_vm_phys_addr_list(struct mm_struct* mm, unsigned long virtual_addr, unsigned long pages);
 extern void* ATI_API_CALL __ke_memset(void* s, int c, __ke_size_t count);
 extern void* ATI_API_CALL __ke_memcpy(void* d, const void* s, __ke_size_t count);
@@ -415,6 +483,7 @@ extern char* ATI_API_CALL __ke_strncpy(char* d, const char* s, __ke_size_t count
 extern int ATI_API_CALL __ke_strcmp(const char *s1, const char *s2);
 extern int ATI_API_CALL __ke_strncmp(const char* str1, const char* str2, __ke_size_t count);
 extern int ATI_API_CALL __ke_sprintf(char* buf, const char* fmt, ...);
+extern int ATI_API_CALL __ke_snprintf(char* buf, size_t size, const char* fmt, ...);
 extern int ATI_API_CALL __ke_vm_test_and_clear_dirty(struct mm_struct* mm, unsigned long virtual_addr);
 extern void* ATI_API_CALL __ke_vmap(unsigned long *pagelist, unsigned int count);
 extern void ATI_API_CALL __ke_vunmap(void* addr);
@@ -423,6 +492,11 @@ extern void ATI_API_CALL __ke_vunmap(void* addr);
 
 extern void ATI_API_CALL __ke_set_bit(int nr, volatile void * addr);
 extern void ATI_API_CALL __ke_clear_bit(int nr, volatile void * addr);
+extern void ATI_API_CALL __ke_change_bit(int nr, volatile void* addr);
+extern int ATI_API_CALL __ke_test_bit(int nr, volatile void* addr);
+extern int ATI_API_CALL __ke_test_and_set_bit(int nr, volatile void* addr);
+extern int ATI_API_CALL __ke_test_and_clear_bit(int nr, volatile void* addr);
+extern int ATI_API_CALL __ke_test_and_change_bit(int nr, volatile void* addr);
 
 /*****************************************************************************/
 
@@ -453,6 +527,8 @@ extern int ATI_API_CALL __ke_pci_write_config_byte(__ke_pci_dev_t* dev, __ke_u8
 extern int ATI_API_CALL __ke_pci_write_config_word(__ke_pci_dev_t* dev, __ke_u8 where, __ke_u16 val);
 extern int ATI_API_CALL __ke_pci_write_config_dword(__ke_pci_dev_t* dev, __ke_u8 where, __ke_u32 val);
 extern int ATI_API_CALL __ke_pci_enable_device(__ke_pci_dev_t* dev);
+extern __ke_pci_dev_t* ATI_API_CALL __ke_pci_find_slot(__ke_u32 bus, __ke_u32 slot);
+extern __ke_u8 ATI_API_CALL __ke_pci_get_busnr(__ke_pci_dev_t* pcidev);
 
 /*****************************************************************************/
 
@@ -494,9 +570,10 @@ extern char* ATI_API_CALL __ke_vm_phys_addr_str(struct vm_area_struct* vma,
                                    unsigned long linear_address, 
                                    __ke_dma_addr_t* phys_address);
 extern int ATI_API_CALL __ke_vm_map(struct file* filp,
-                       struct vm_area_struct* vma,
-                       enum __ke_vm_maptype type,
-                       int readonly);
+                                    struct vm_area_struct* vma,
+                                    unsigned long offset,
+                                    enum __ke_vm_maptype type,
+                                    int readonly);
 
 #ifdef __x86_64__
 /*
@@ -537,7 +614,7 @@ int ATI_API_CALL __ke_agp_available(__ke_pci_dev_t *pcidev, int use_internal);
 void ATI_API_CALL __ke_agp_uninit(void);
 #ifdef FGL
 struct _agp_memory* ATI_API_CALL __ke_agp_allocate_memory_phys_list(
-    __ke_size_t pages, unsigned long type, unsigned long * phys_addr);
+    __ke_size_t pages, unsigned long type, __ke_dma_addr_t * phys_addr);
 #endif
 void ATI_API_CALL __ke_agp_free_memory(struct _agp_memory* handle);
 struct _agp_memory* ATI_API_CALL __ke_agp_allocate_memory(__ke_size_t pages, 
@@ -580,18 +657,8 @@ unsigned long ATI_API_CALL firegl_agp_memory_page_count(struct _agp_memory* hand
 int ATI_API_CALL __ke_agp_memory_get_page_count(struct _agp_memory* agpmem);
 void ATI_API_CALL __ke_agp_memory_get_memory(struct _agp_memory* agpmem,
                                 unsigned long **memory_ptr);
-unsigned long ATI_API_CALL firegl_igp_get_agpmem_phys_addr(
-                                                void*         drm_filp,
-                                                unsigned long virtual_addr,
-                                                unsigned long vm_start,
-                                                unsigned long vm_end,
-                                                unsigned long handle);
-int ATI_API_CALL firegl_is_igp_rs400(void *priv);
-int ATI_API_CALL firegl_is_igp_rs480(void *priv);
 
 #ifdef __x86_64__
-int ATI_API_CALL firegl_call_ioctl(unsigned int fd, unsigned int cmd, void *data);
-int ATI_API_CALL __ke_sys_ioctl(unsigned int fd, unsigned int cmd, void *data);
 
 int ATI_API_CALL firegl_get_user_ptr(__ke_u32 *src,   void **dst);
 int ATI_API_CALL firegl_get_user_u16(__ke_u16 *src,   __ke_u16 *dst);
@@ -602,6 +669,9 @@ int ATI_API_CALL firegl_put_user_ptr(void *src,      __ke_u32 *dst);
 int ATI_API_CALL firegl_put_user_u16(__ke_u16 src,   __ke_u16 *dst);
 int ATI_API_CALL firegl_put_user_u32(__ke_u32 src,   __ke_u32 *dst);
 int ATI_API_CALL firegl_put_user_u64(__ke_u64 src,   __ke_u32 *dst);
+int ATI_API_CALL firegl_init_32compat_ioctls(void);
+void ATI_API_CALL firegl_kill_32compat_ioctls(void);
+
 #endif
 
 /*****************************************************************************/
@@ -611,6 +681,13 @@ extern void ATI_API_CALL __ke_pci_unmap_single (__ke_pci_dev_t *pdev, __ke_dma_a
 extern __ke_dma_addr_t ATI_API_CALL __ke_pci_map_page (__ke_pci_dev_t *pdev, unsigned long buffer, unsigned long offset, __ke_size_t size, int direction);
 extern void ATI_API_CALL __ke_pci_unmap_page (__ke_pci_dev_t *pdev, __ke_dma_addr_t bus_addr, __ke_size_t size, int direction); 
 extern int ATI_API_CALL __ke_log2(unsigned long x);
+extern __ke_u8 ATI_API_CALL __ke_readb(void* p, __ke_u32 offset);
+extern int firegl_xServer_lock(struct inode* inode __attribute__((unused)), struct file* filp, unsigned int cmd __attribute__((unused)), unsigned long arg);
+extern int firegl_init_asic(struct inode* inode __attribute__((unused)), struct file* filp, unsigned int cmd __attribute__((unused)), unsigned long arg);
+extern int ATI_API_CALL firegl_pci_save_state(struct drm_device* dev);
+extern int ATI_API_CALL firegl_pci_restore_state(struct drm_device* dev);
+extern int ATI_API_CALL firegl_powerdown(struct drm_device* dev, __ke_u32 state);
+extern int ATI_API_CALL firegl_powerup(struct drm_device* dev);
 
 
 #define __ke_PCI_DMA_BIDIRECTIONAL  0
@@ -628,6 +705,13 @@ extern int __ke_debuglevel;
 
 extern int __ke_moduleflags;
 
+#define AGPGART_INUSE_NONE		0x00000000
+#define KERNEL26_AGPGART_INUSE		0x00000001
+#define KERNEL24_AGPGART_INUSE		0x00000002
+#define INTERNAL_AGPGART_INUSE		0x00000004
+
+extern unsigned int __ke_firegl_agpgart_inuse;
+
 /* module flags*/
 #define __KE_FLAG_DEBUG                     0x00000001
 #define __KE_FLAG_NOAUTH                    0x00000002
@@ -636,6 +720,12 @@ extern int __ke_moduleflags;
 #define __KE_FLAG_DISABLE_FIREGL_AGPLOCK    0x00000010
 #define __KE_FLAG_DISABLE_DYNAMIC_PCIE      0x00000020
 
+#define __KE_PM_SUSPEND_ON                  0
+#define __KE_PM_SUSPEND_STANDBY             1
+#define __KE_PM_SUSPEND_MEM                 3
+#define __KE_PM_SUSPEND_DISK                4
+#define __KE_PM_SUSPEND_MAX                 5
+
 /* global constants */
 extern const char*          __ke_UTS_RELEASE;
 extern const unsigned long  __ke_LINUX_VERSION_CODE;
@@ -737,5 +827,14 @@ extern unsigned long        __ke_HZ;
 #endif // !__KE_NO_VSPRINTF
 
 /*****************************************************************************/
+#ifdef _KE_SERIAL_DEBUG
+extern int  ATI_API_CALL __ke_SerPrint(const char *fmt, ...);
+extern void ATI_API_CALL __ke_SetSerialPort(void);
+extern void ATI_API_CALL __ke_printstr(const char *str);
+#endif
+
+#ifndef __GFP_COMP
+#define __GFP_COMP 0
+#endif
 
 #endif /* _FIREGL_PUBLIC_H_ */
