@@ -8,7 +8,7 @@
 # ==============================================================
 # local variables and files
 current_wd=`pwd`
-logfile=$current_wd/make.log
+logfile=$current_wd/make.sh.log	# DKMS uses name 'make.log', so that we need another name
 
 # project name
 MODULE=fglrx
@@ -32,13 +32,6 @@ CHECK_P3=0
 
 # default options
 OPTIONS_HINTS=1
-if [ -z "${CC}" ]; then 
-	CC=gcc
-fi
-# use below lines to force a specific compiler version
-# on systems that do have multiple compilers with a non-matching default
-#CC=gcc -V 2.96
-#CC=gcc -V 3.2
 
 # sets the GCC to use to the one required by the module (if available)
 function set_GCC_version () {
@@ -51,11 +44,11 @@ function set_GCC_version () {
   #see if they don't match
   if [ ${GCC_MAJOR} != ${KERNEL_GCC_MAJOR} ]; then
        #use kernel GCC version hopefully
-       KERNEL_GCC="`cat /proc/version | grep -o -e "gcc version .* " | cut -d " " -f 3`"
-       CC="gcc -V ${KERNEL_GCC}"
+       KERNEL_GCC="`cat /proc/version | grep -o -e "gcc version [0-9]\.[0-9]" | cut -d " " -f 3`"
+       CC="gcc-${KERNEL_GCC}"
 
        # check if gcc version requested exists
-       GCC_AVAILABLE="`${CC} --version | grep -e "(GCC)" |cut -d " " -f 3`"
+       GCC_AVAILABLE="`${CC} --version | grep -e "(GCC)" |cut -d " " -f 3 | cut -c-3`"
     
        if [ ${GCC_AVAILABLE} != ${KERNEL_GCC} ]; then
            echo "The GCC version that is required to compile this module is version ${KERNEL_GCC}."
@@ -65,7 +58,10 @@ function set_GCC_version () {
   fi
 }
 
-set_GCC_version
+if [ -z "${CC}" ]; then 
+	CC=gcc
+	set_GCC_version
+fi
 
 # parse options
 if [ "$1" = "--nohints" ]; then
@@ -239,7 +235,7 @@ if [ "$OsRelease" = "$kernel_release" ];
 then
   hit=1
 else
-  # RedHat 7.0 source and some newer 2.4.x might not have smp suffix in UTS_RELEASE text
+  # Red Hat 7.0 source and some newer 2.4.x might not have smp suffix in UTS_RELEASE text
   if [ `echo $OsRelease | grep smp -c` -ne 0 ];
   then
     if [ "$OsRelease" = "${kernel_release}smp" ];
@@ -347,7 +343,7 @@ src_file=$linuxincludes/linux/agp_backend.h
 if [ -e $src_file ];
 then
   AGP=1
-  def_agp=-D__AGP__
+#  def_agp=-D__AGP__
   echo "file $src_file says: AGP=$AGP"                             >> $logfile
 fi
 
@@ -527,7 +523,7 @@ fi
 # act on final result
 if [ ! $MODVERSIONS = 0 ]
 then
-  def_modversions="-DMODVERSIONS -include $linuxincludes/linux/modversions.h"
+  def_modversions="-DMODVERSIONS"
 fi
 
 
@@ -668,10 +664,10 @@ then
   rm -f ${MODULE}.o     2>&1 | tee -a $logfile
 fi
 
-#if [ -e agpgart_fe.o ]
-#then 
-#  rm -f agpgart_fe.o 2>&1 | tee -a $logfile
-#fi
+if [ -e agpgart_fe.o ]
+then 
+  rm -f agpgart_fe.o 2>&1 | tee -a $logfile
+fi
 if [ -e agpgart_be.o ]
 then 
   rm -f agpgart_be.o 2>&1 | tee -a $logfile
@@ -845,8 +841,9 @@ fi
 if [ $kernel_is_26x -gt 0 ]; then
     echo "doing Makefile based build for kernel 2.6.x and higher"   | tee -a $logfile
     cd 2.6.x
+    V=${V:-0}
     #tlog is a tempory file that will be deleted
-    make PAGE_ATTR_FIX=$PAGE_ATTR_FIX > tlog 2>&1 
+    make CC=${CC} V=${V} MODFLAGS="-DMODULE $def_for_all $def_smp $def_modversions" PAGE_ATTR_FIX=$PAGE_ATTR_FIX > tlog 2>&1 
     res=$?
     tee -a $logfile < tlog
     #delete tlog
@@ -863,6 +860,29 @@ if [ $kernel_is_26x -gt 0 ]; then
     fi
     ln -s 2.6.x/${MODULE}${kmod_extension}
     TERMINAL_HINT=0
+
+
+#	# make fglrx_agp.ko
+#    echo " compiling fglrx_agp.ko module"
+#    cd firegl_agpgart
+#    V=${V:-0}
+#    #tlog is a tempory file that will be deleted
+#    make V=${V} MODFLAGS="-DMODULE $def_for_all $def_smp $def_modversions" PAGE_ATTR_FIX=$PAGE_ATTR_FIX > tlog 2>&1 
+#    res=$?
+#    tee -a $logfile < tlog
+#    #delete tlog
+#    rm -f tlog
+#    cd ..
+#    if [ $res -eq 0 ]; then
+#        echo "AGPGART build succeeded with return value $res"               | tee -a $logfile
+#	    if [ -e ${MODULE_agpgart}${kmod_extension} ]; then
+#	         rm -f ${MODULE_agpgart}${kmod_extension}
+#	    fi
+#	    echo " finished compiling for fglrx_agp"
+#	    ln -s firegl_agpgart/${MODULE_agpgart}${kmod_extension}
+#    else
+#        echo "AGPGART module build failed with return value $res"                  | tee -a $logfile
+#    fi
 else
     echo "doing script based build for kernel 2.4.x and similar"    | tee -a $logfile
     
@@ -975,7 +995,6 @@ echo "linking of ${MODULE} kernel module..." | tee -a $logfile
 if [ ! -z "${MODULE_NAME}" ]; then 
 	module_version=.${MODULE_NAME}
 fi
-#ld="ld -r ${FGL_PUBLIC}_public.o agpgart_fe.o agpgart_be.o agp3.o i7505-agp.o nvidia-agp.o $core_lib -o ${MODULE}.o"
 ld="ld -r ${FGL_PUBLIC}_public.o agpgart_be.o agp3.o i7505-agp.o nvidia-agp.o $core_lib -o ${MODULE}${module_version}.o"
 echo "$ld" >> $logfile
 $ld 2>&1 | tee -a $logfile
@@ -1012,7 +1031,20 @@ then
     echo "creating target directory" >> $logfile
     mkdir $target_dir | tee -a $logfile
   fi
-  
+
+  which strip > /dev/null 2>&1
+  if test $? = 0; then
+     strip -g ${MODULE}${kmod_extension} > /dev/null 2>&1
+     if test $? = 0; then	
+        echo "stripping the debug info of kernel module" >> $logfile
+     else
+        echo "could not strip the debug info of kernel module" >> $logfile
+     fi 
+  else
+     echo "could not find the strip utility on your system" >> $logfile
+  fi
+
+  # for fglrx and fglrx_agp
   echo "copying ${MODULE}${kmod_extension}" >> $logfile
   if [ $INSTALL_MODULE_SUBDIRS -eq 1 ];
   then
