@@ -50,6 +50,7 @@
 #include <stdlib.h>
 #include <err.h>
 
+#include <net/if.h>
 
 static int
 setsysctrl(const char *dev, const char *control , u_long value)
@@ -85,7 +86,7 @@ static void usage(void)
 int
 main(int argc, char *argv[])
 {
-	char device[5];
+	char device[IFNAMSIZ];
 	int distance = -1;
 	int c;
 
@@ -119,6 +120,7 @@ main(int argc, char *argv[])
 		
 		printf("Setting distance on interface %s to %i meters\n", device, distance);
 		setsysctrl(device, "slottime", slottime);
+		setsysctrl(device, "lockslottime", 1);
 		setsysctrl(device, "acktimeout", acktimeout);
 		setsysctrl(device, "ctstimeout", ctstimeout);
 	}else{
